############################################################################
##
## Copyright (c) 2000-2001 BalaBit IT Ltd, Budapest, Hungary
## All rights reserved.
##
## $Id: InstanceConfig.py,v 1.7 2002/10/22 11:01:50 sasa Exp $
##
############################################################################

import UIUtils
from Config import Config
from ServiceConfig import ServiceConfig
from ListenerConfig import ListenerConfig
from Predef import PredefDB
import Predef
from snack import *
from Lang import *

class InstanceConfig(Config):
    def __init__(self, owner, name):
        self.owner = owner
        self.name = name
        self.services = {}
        self.listeners = []
                
    def importHook(self, output, gi):
        for x in self.services.keys():
            self.services[x].importHook(output, gi)

        for x in self.listeners:
            x.importHook(output, gi)
                        
    def globalHook(self, output, gi):
        for x in self.services.keys():
            self.services[x].globalHook(output, gi)
        for x in self.listeners:
            x.globalHook(output, gi)
                
    def generateConfig(self, output, gi):
        info = {
            "name": self.name
            }
        output.write('def %(name)s():\n\n' % info)
        gi.indent_level = gi.indent_level + 1
                
        body = 0
        for x in self.services.keys():
            body = 1
            self.services[x].generateConfig(output, gi)
                        
        for x in self.listeners:
            body = 1
            x.generateConfig(output, gi)
                
        if not body:
            output.write('\t' * gi.indent_level + 'pass\n\n')
        else:
            output.write('\n')
            
        gi.indent_level = gi.indent_level - 1

    def addService(self):
        x = UIUtils.EntryForm(title=mylang.get("Service name"), text="%s_HTTP" % self.name).run()
        if x:
	    if self.services.has_key(x):
	        UIUtils.MessageForm(title=mylang.get("Message"), text=mylang.get("This service (%s) already exists.") % x).run()
		return

            self.services[x] = ServiceConfig(self, x)
            self.services_lb.append(x, self.services[x])
        
    def editService(self):
        try:
            self.services_lb.current().runForm()
        except KeyError:
            pass

    def delService(self):
        try:
            x = self.services_lb.current()
            if x:
                del self.services[x.name]
                self.services_lb.delete(x)
        except KeyError:
            pass


    def addListener(self):
        from ListenerConfig import prev_listener
        
        x = ListenerConfig(self, prev_listener)
        self.listeners.append(x)
        self.listeners_lb.append(x.addr[-20:], x)
        self.listeners_lb.setCurrent(x)
        self.editListener()
        
    def editListener(self):
        try:
            l = self.listeners_lb.current()
            l.runForm()
            self.listeners_lb.replace(l.addr[-20:], l)
        except KeyError, e:
            pass

    def delListener(self):
        try:
            x = self.listeners_lb.current()
            if x:
                self.listeners.remove(x)
                self.listeners_lb.delete(x)
        except KeyError:
            pass

    def predefService(self):
        Predef.predefdb.runForm(self)

    def runForm(self):
        screen = UIUtils.screen
        form = GridForm(screen, mylang.get("Edit instance %s") % self.name, 1, 2)
                
        uppergrid = Grid(4, 2)
        
        self.services_lb = services = Listbox(5, width=20, scroll=1, border=1)
        services_l = Label(mylang.get("Services:"))
        services_bb = ButtonBar(screen, 
                                [(mylang.get(" Add  "), "add"), (mylang.get(" Edit "), "edit"), (mylang.get("Delete"), "delete")],
                                compact=1, horizontal=0)
                                        
        for x in self.services.keys():
            services.append(x, self.services[x])
                
        self.listeners_lb = listeners = Listbox(5, width=30, scroll=1, border=1)
        listeners_l = Label(mylang.get("Listeners:"))
        listeners_bb = ButtonBar(screen, 
                                 [(mylang.get(" Add  "), "add"), (mylang.get(" Edit "), "edit"), (mylang.get("Delete"), "delete")],
                                 compact=1, horizontal=0)
                                         
        for x in self.listeners:
            listeners.append(x.addr[-20:], x)
                
        uppergrid.setField(services_l, 0, 0)
        uppergrid.setField(services, 0, 1)
        uppergrid.setField(services_bb, 1, 1)
        
        uppergrid.setField(listeners_l, 2, 0)
        uppergrid.setField(listeners, 2, 1)
        uppergrid.setField(listeners_bb, 3, 1)
                
        main_bb = ButtonBar(screen, [(mylang.get("Predefined"), "predef"), (mylang.get("Close"), "close")])

        form.add(uppergrid, 0, 0)
        form.add(main_bb, 0, 1)
                
        quit = 0
        while not quit:
            x = form.run()
                
            services_action = services_bb.buttonPressed(x)
            listeners_action = listeners_bb.buttonPressed(x)
            main_action = main_bb.buttonPressed(x)
            
            if main_action:
                if main_action == "close":
                    quit = 1
                elif main_action == "predef":
                    self.predefService()
            elif services_action:
                if services_action == "add":
                    self.addService()
                elif services_action == "edit":
                    self.editService()
                elif services_action == "delete":
                    self.delService()
            elif listeners_action:
                if listeners_action == "add":
                    self.addListener()
                elif listeners_action == "edit":
                    self.editListener()
                elif listeners_action == "delete":
                    self.delListener()

        screen.popWindow()
        del self.services_lb
        del self.listeners_lb
                        
