#!/usr/bin/env python
############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## All rights reserved.
##
## $Id: Main.py,v 1.9 2002/10/22 11:01:50 sasa Exp $
##
############################################################################

import sys
from snack import *
import cPickle
import traceback
from ZUI import ZorpConfig
from ZUI import Predef
from ZUI import UIUtils
from ZUI.Params import *
from ZUI.Lang import *

class MainForm:
    def __init__(self, firewalls):
        self.form = None
        self.firewalls = firewalls
        self.initForm()

    def initForm(self):
        fwlabel = Label(mylang.get("Firewalls:"))
	self.fw = fw = Listbox(height = 7, width = 30, returnExit = 1, scroll = 1, border = 1)

        for f in self.firewalls.keys():
            self.fw.append(f, self.firewalls[f])
            
	self.bb = bb = ButtonBar(UIUtils.screen, 
		       [(mylang.get(" Add  "), "add"), 
			(mylang.get(" Edit "), "edit"), 
			(mylang.get("Delete"), "delete"), 
			(mylang.get(" Save "), "save"),
			(mylang.get(" Exit "), "exit")], 
		       horizontal=0,
		       compact=1)

	self.form = GridForm(UIUtils.screen, mylang.get("Choose a firewall"), 2, 4)
	self.form.add(fwlabel, 0, 0, anchorLeft = 1)
	self.form.add(fw, 0, 1, padding = (0, 1, 0, 1))
	self.form.add(bb, 1, 1, growy=1)

    def add(self, app):
        x = UIUtils.EntryForm(title=mylang.get("Firewall name"), text="zorp@fw").run()
        if x:
            if not self.firewalls.has_key(x):
                self.firewalls[x] = ZorpConfig.ZorpConfig(x)
                self.fw.append(x, self.firewalls[x])
            else:
                UIUtils.MessageForm(title=mylang.get("Message"), text=mylang.get("This firewall name already exists.")).run()
        return 1

    def edit(self, app):
        try:
            fw = self.fw.current()
            if fw:
                x = fw.runForm()
        except KeyError:
            pass
        return 1

    def delete(self, app):
        try:
            fw = self.fw.current()
        except KeyError:
            fw = None
        if fw:
            self.fw.delete(fw)
            del self.firewalls[fw.name]
        return 1
        
    def save(self, app):
        app.save()
        return 1

    def exit(self, app):
        return 0

    def run(self, app):

        res = self.form.run()
	
	action = self.bb.buttonPressed(res)

	if action == None:
	    action = "edit"

        f = getattr(self, action)
        return f(app)
	    

class ZorpUIApp:
    def __init__(self):
        UIUtils.Initialize("zui version %s" % (version,))
        Predef.predefdb = Predef.PredefDB(path_predefined_db)
        try:
            self.fwdbfile = sys.argv[1]
        except:
            self.fwdbfile = path_zui_db
        self.load()
        

    def load(self):
        try:
            f = open(self.fwdbfile, 'r')
            self.firewalls = cPickle.load(f)
            f.close()
        except IOError:
            self.firewalls = {}

    def save(self):


        x = UIUtils.EntryForm(title=mylang.get("ZUI database file"), text=self.fwdbfile).run()
        if x:
            self.fwdbfile = x
        try:
            f = open(self.fwdbfile, 'w')
            cPickle.dump(self.firewalls, f)
            f.close()
        except IOError:
            UIUtils.MessageForm(mylang.get("Warning"), mylang.get("An I/O error occurred while saving firewall database.")).run()
            return 0
        return 1

    def run(self):
        if not Predef.predefdb:
            return
        mainform = MainForm(self.firewalls)
        quit = 0
        while not quit:
	    while mainform.run(self):
                pass

            res = UIUtils.YesNoForm(mylang.get("Do you want to save changes?")).run()
            if res == "yes":
                quit = self.save()
            else:
                quit = 1




def main():
    try:
        app = ZorpUIApp()
        app.run()
        UIUtils.Deinitialize()
    except:
        UIUtils.Deinitialize()
        traceback.print_exc()

if __name__ == "__main__":
    main()
